==> dolfin-0.9.2.tar.gz: dolfin-0.9.2/site-packages/dolfin/specialfunctions.py <==
# -*- coding: utf-8 -*-
"""This module defines some special functions (originally defined in
SpecialFunctions.h) like MeshSize and FacetNormal."""

__author__ = "Anders Logg (logg@simula.no)"
__date__ = "2008-12-08 -- 2009-03-11"
__copyright__ = "Copyright (C) 2008-2009 Anders Logg"
__license__  = "GNU LGPL Version 2.1"

__all__ = ["MeshCoordinates", "FacetNormal", "MeshSize", "AvgMeshSize", 
           "FacetArea", "InvFacetArea","IsOutflowFacet", "SUPGStabilizer"]

# Import FFC and SWIG-generated extension module (DOLFIN C++)
import ffc
import cpp

# Local imports
from functionspace import dim2domain, FunctionSpaceBase
from function import Function

class MeshCoordinates(ffc.Function, cpp.MeshCoordinates, Function):
    def __init__(self, mesh):
        "Create function that evaluates to the mesh coordinates at each vertex."

        domain = dim2domain[mesh.topology().dim()]
        element = ffc.VectorElement("Lagrange", domain, 1)

        self._V = FunctionSpaceBase(mesh, element)

        ffc.Function.__init__(self, element)
        cpp.MeshCoordinates.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)

class FacetNormal(ffc.Function, cpp.FacetNormal, Function):

    def __init__(self, mesh):
        "Create function that evaluates to the facet normal on each facet."

        domain = dim2domain[mesh.topology().dim()]
        element = ffc.VectorElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)
        
        ffc.Function.__init__(self, element)
        cpp.FacetNormal.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)


class MeshSize(ffc.Function, cpp.MeshSize, Function):
    def __init__(self, mesh):
        "Create function that evaluates to the local mesh size on each cell."

        domain = dim2domain[mesh.topology().dim()]
        element = ffc.FiniteElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)

        ffc.Function.__init__(self, element)
        cpp.MeshSize.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)


class AvgMeshSize(ffc.Function, cpp.AvgMeshSize, Function):

    def __init__(self, mesh):
        """Create function that evaluates to the average local mesh size
        over the two neighboring cells on each facet."""

        domain = dim2domain[mesh.topology().dim()]
        element = ffc.FiniteElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)

        ffc.Function.__init__(self, element)
        cpp.AvgMeshSize.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)


class FacetArea(ffc.Function, cpp.FacetArea, Function):

    def __init__(self, mesh):
        "Create function that evaluates to the facet area/length on each facet."

        domain = dim2domain[mesh.topology().dim()]
        element = ffc.FiniteElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)

        ffc.Function.__init__(self, element)
        cpp.FacetArea.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)


class InvFacetArea(ffc.Function, cpp.InvFacetArea, Function):

    def __init__(self, mesh):
        "Create function that evaluates to the inverse facet area/length on each facet."

        domain = dim2domain[mesh.topology().dim()]
        element = ffc.FiniteElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)

        ffc.Function.__init__(self, element)
        cpp.InvFacetArea.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)


class IsOutflowFacet(ffc.Function, cpp.IsOutflowFacet, Function):
    def __init__(self, field):
        """ Create a function that evaluates to 1.0 if the current facet
        is an outflow facet with respect to the current cell and a given
        vector field and 0.0 otherwise"""
        assert isinstance(field,cpp.Function), "Provide a cpp.Function for 'field'"
        assert isinstance(field,ffc.Function), "Provide a ffc.Function for 'field'"
        mesh = field.function_space().mesh() 
        
        domain = dim2domain[mesh.topology().dim()]
        element = ffc.FiniteElement("Discontinuous Lagrange", domain, 0)
        
        self._V = FunctionSpaceBase(mesh, element)
        
        ffc.Function.__init__(self, element)
        cpp.IsOutflowFacet.__init__(self, self._V, field)
    
    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)

class SUPGStabilizer(ffc.Function, cpp.SUPGStabilizer, Function):
    def __init__(self, field, sigma=1.0):
        """ Creates a Streamline Petrov-Galerkin stabilizing function
        
        Given the advective field a, this function computes the stabilizing factor
       
                  s = h*tau*a/(2*|a|)
       
        where h is the local size of the mesh, tau the local stabilizing factor
        calculated from the local Péclet number, a the advective field.
        
        @param field : The field driving the advection
        @param sigma : The diffusion coefficient defaults to 1.0
                       
        """
        assert isinstance(field,cpp.Function), "Provide a cpp.Function for 'field'"
        assert isinstance(field,ffc.Function), "Provide a ffc.Function for 'field'"
        assert isinstance(sigma,(float,int)),  "Provide a scalar for 'sigma'"
        
        mesh = field.function_space().mesh() 
        
        domain = dim2domain[mesh.topology().dim()]
        element = ffc.VectorElement("Discontinuous Lagrange", domain, 0)
        
        self._V = FunctionSpaceBase(mesh, element)
        
        ffc.Function.__init__(self, element)
        cpp.SUPGStabilizer.__init__(self, self._V, field, sigma)
    
    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ffc.Function.__repr__(self)

==> dolfin-0.9.3.tar.gz: dolfin-0.9.3/site-packages/dolfin/specialfunctions.py <==
# -*- coding: utf-8 -*-
"""This module defines some special functions (originally defined in
SpecialFunctions.h) like MeshSize and FacetNormal."""

__author__ = "Anders Logg (logg@simula.no)"
__date__ = "2008-12-08 -- 2009-03-11"
__copyright__ = "Copyright (C) 2008-2009 Anders Logg"
__license__  = "GNU LGPL Version 2.1"

__all__ = ["MeshCoordinates", "FacetNormal", "CellSize", "FacetArea", 
           "SUPGStabilizer"]

# Import UFL and SWIG-generated extension module (DOLFIN C++)
import ufl
import cpp

# Local imports
from functionspace import dim2domain, FunctionSpaceBase
from function import Function

class MeshCoordinates(Function, ufl.Function, cpp.MeshCoordinates):
    def __init__(self, mesh):
        "Create function that evaluates to the mesh coordinates at each vertex."

        domain = dim2domain[mesh.topology().dim()]
        element = ufl.VectorElement("Lagrange", domain, 1)

        self._V = FunctionSpaceBase(mesh, element)

        ufl.Function.__init__(self, element)
        cpp.MeshCoordinates.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ufl.Function.__repr__(self)

class FacetNormal(Function, ufl.Function, cpp.FacetNormal):

    def __init__(self, mesh):
        "Create function that evaluates to the facet normal on each facet."

        domain = dim2domain[mesh.topology().dim()]
        element = ufl.VectorElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)
        
        ufl.Function.__init__(self, element)
        cpp.FacetNormal.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ufl.Function.__repr__(self)


class CellSize(Function, ufl.Function, cpp.CellSize):
    def __init__(self, mesh):
        "Create function that evaluates to the local cell size."

        domain = dim2domain[mesh.topology().dim()]
        element = ufl.FiniteElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)

        ufl.Function.__init__(self, element)
        cpp.CellSize.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ufl.Function.__repr__(self)


class FacetArea(Function, ufl.Function, cpp.FacetArea):

    def __init__(self, mesh):
        "Create function that evaluates to the facet area/length on each facet."

        domain = dim2domain[mesh.topology().dim()]
        element = ufl.FiniteElement("Discontinuous Lagrange", domain, 0)

        self._V = FunctionSpaceBase(mesh, element)

        ufl.Function.__init__(self, element)
        cpp.FacetArea.__init__(self, self._V)

    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ufl.Function.__repr__(self)


class SUPGStabilizer(Function, ufl.Function, cpp.SUPGStabilizer):
    def __init__(self, field, sigma=1.0):
        """ Creates a Streamline Petrov-Galerkin stabilizing function
        
        Given the advective field a, this function computes the stabilizing factor
       
                  s = h*tau*a/(2*|a|)
       
        where h is the local size of the mesh, tau the local stabilizing factor
        calculated from the local Péclet number, a the advective field.
        
        @param field : The field driving the advection
        @param sigma : The diffusion coefficient defaults to 1.0
                       
        """
        assert isinstance(field,cpp.Function), "Provide a cpp.Function for 'field'"
        assert isinstance(field,ufl.Function), "Provide a ufl.Function for 'field'"
        assert isinstance(sigma,(float,int)),  "Provide a scalar for 'sigma'"
        
        mesh = field.function_space().mesh() 
        
        domain = dim2domain[mesh.topology().dim()]
        element = ufl.VectorElement("Discontinuous Lagrange", domain, 0)
        
        self._V = FunctionSpaceBase(mesh, element)
        
        ufl.Function.__init__(self, element)
        cpp.SUPGStabilizer.__init__(self, self._V, field, sigma)
    
    def function_space(self):
        " Return the function space."
        return self._V

    def __str__(self):
        return "<Function in %s>" % str(self._V)
    
    def __repr__(self):
        return ufl.Function.__repr__(self)
